// Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 1733 $
// $LastChangedDate:: 2019-08-29 09:41:24 +0200#$
// 
// This function returns the port label style array suitable for the Xcos JGraphX 'in_style' and 'out_style' property.
// It returns an empty string if the source port label array does not contain elements.
// Example: [new_port_label] = getPortLabelStyle(port_label)
// 
// Parameters:
// port_label ... Source/existing port label array
// 
// Return values:
// new_port_label ... Port label array

function [new_port_label] = getPortLabelStyle(port_label)
    funcprot(0);

    // return empty string in case of empty (= no port) label
    no_ports = size(port_label, 1);
    if no_ports == 0 then
        new_port_label = "";
        return;
    end

    // default inport- & outport style
    base_struct = struct();
    base_struct.shape = "triangle";
    base_struct.fillColor = "black";
    base_struct.noLabel = "false";
    
    for (i = 1 : no_ports)
        cur_port_struct = mergeStructures(base_struct, string2struct(port_label(i)));
        if isfield(cur_port_struct, "textOpacity") then
            opacity_value = strtod(cur_port_struct.textOpacity);
            if opacity_value == 0 then
                // opacity = 0 => transparency = 100% => not visible => remove text opacity field and set field 'noLabel' to true
                cur_port_struct.textOpacity = null();
                cur_port_struct.noLabel = "true";
            else
                cur_port_struct.noLabel = "false";
            end
        end
        new_port_label(i, 1) = struct2string(cur_port_struct);
    end
endfunction
