/*
 * HotInt specific configuration.
 * 
 * Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include <math.h>
#include "X2C.h"
#include "HotInt.h"

/* private prototypes */
static uint8 linkPort(char* name, uint32 linkPosition);
static uint8 setInportValue(uint32 tablePosition, float64 value);
static uint8 getOutportValue(uint32 tablePosition, float64* value);

/* CODEGEN-BEGIN:External variables */
/* CODEGEN-END:External variables */

/* CODEGEN-BEGIN:Port Amount */
/* CODEGEN-END:Port Amount */
static const uint32 noIoPorts = noInports + noOutports;

/* CODEGEN-BEGIN:Sample Time Info */
/* CODEGEN-END:Sample Time Info */
static const double updateInterval = sampleTime * calculationTimeFactor;

static const char* ioPortNames[noIoPorts] = {
/* CODEGEN-BEGIN:IO Port Names */
/* CODEGEN-END:IO Port Names */
};

/**
 * port types
 * 0b 0000 0000
 *    +-------- --- isInteger bit (true = integer, false = double value)
 *     +------- --- isSigned bit (true = signed, false = unsigned value)
 *      ++ ++++ --- byte count (data type byte count, e.g. 32-bit = 4, 64-bit = 8 etc.)
 */
static const uint8 ioPortTypes[noIoPorts] = {
/* CODEGEN-BEGIN:IO Port Types */
/* CODEGEN-END:IO Port Types */
};

/* contains pointer to inport- and outport data */
static void* ioPortData[noIoPorts];

/**
 * links order of retrieved values to order of ioPortData
 * E.g. 2nd retrieved value = 3rd value in ioPortData
 * => linkTable[1] = 2
 * => 2nd retrieved value -> lookup linkTable[1] = 2 -> use pointer to value ioPortData[2]
 */
static uint32 linkTable[noIoPorts];

static const double ioPortGain[noIoPorts] = {
/* CODEGEN-BEGIN:IO Port Gain */
/* CODEGEN-END:IO Port Gain */
};

static const double ioPortOffset[noIoPorts] = {
/* CODEGEN-BEGIN:IO Port Offset */
/* CODEGEN-END:IO Port Offset */
};

void initPortArray(void)
{
/* CODEGEN-BEGIN:IO Port Links */
/* CODEGEN-END:IO Port Links */
}

float64 getUpdateInterval(void)
{
	return (updateInterval);
}

static uint8 linkPort(char* name, uint32 linkPosition)
{
	uint8 error = 0;
	uint32 curNamePos = 0;
	uint8 found = 0;

	while (!found && (curNamePos < noIoPorts))
	{
		if (strcmp(name, ioPortNames[curNamePos]) == 0)
		{
			linkTable[linkPosition] = curNamePos;
			found = 1;
		}
		curNamePos++;
	}

	if (found)
	{
		error = 0;
	}
	else
	{
		error = 1;
	}
	return (error);
}

/* sets inport value */
static uint8 setInportValue(uint32 tablePosition, float64 value)
{
	uint8 error = 0;
	uint8 isInteger, isSigned, byteCount;
	float64 tmpValue, offset, gain;

	uint32 linkPosition = linkTable[tablePosition];
	
	isInteger = (ioPortTypes[linkPosition] >> 7) & 0x01;
	isSigned = (ioPortTypes[linkPosition] >> 6) & 0x01;
	byteCount = (ioPortTypes[linkPosition] & 0x3F);

	offset = ioPortOffset[linkPosition];
	gain = ioPortGain[linkPosition];
	tmpValue = (value + offset) * gain;
	 
	if (isInteger)
	{
		tmpValue = round(tmpValue);
		switch (byteCount)
		{
		case 1:
			{
				if (isSigned)
				{
					int8 tmp8;
					if (tmpValue > INT8_MAX)
					{
						tmp8 = INT8_MAX;
					}
					else if (tmpValue < INT8_MIN)
					{
						tmp8 = INT8_MIN;
					}
					else
					{
						tmp8 = (int8)tmpValue;
					}
					*(int8*)ioPortData[linkPosition] = tmp8;
				}
				else
				{
					uint8 tmp8;
					if (tmpValue > UINT8_MAX)
					{
						tmp8 = UINT8_MAX;
					}
					else if (tmpValue < 0)
					{
						tmp8 = 0;
					}
					else
					{
						tmp8 = (uint8)tmpValue;
					}
					*(uint8*)ioPortData[linkPosition] = tmp8;
				}
				break;
			}
		case 2:
			{
				if (isSigned)
				{
					int16 tmp16;
					if (tmpValue > INT16_MAX)
					{
						tmp16 = INT16_MAX;
					}
					else if (tmpValue < INT16_MIN)
					{
						tmp16 = INT16_MIN;
					}
					else
					{
						tmp16 = (int16)tmpValue;
					}
					*(int16*)ioPortData[linkPosition] = tmp16;
				}
				else
				{
					uint16 tmp16;
					if (tmpValue > UINT16_MAX)
					{
						tmp16 = UINT16_MAX;
					}
					else if (tmpValue < 0)
					{
						tmp16 = 0;
					}
					else
					{
						tmp16 = (uint16)tmpValue;
					}
					*(uint16*)ioPortData[linkPosition] = tmp16;
				}
				break;
			}
		case 4:
			{
				if (isSigned)
				{
					int32 tmp32;
					if (tmpValue > INT32_MAX)
					{
						tmp32 = INT32_MAX;
					}
					else if (tmpValue < INT32_MIN)
					{
						tmp32 = INT32_MIN;
					}
					else
					{
						tmp32 = (int32)tmpValue;
					}
					*(int32*)ioPortData[linkPosition] = tmp32;
				}
				else
				{
					uint32 tmp32;
					if (tmpValue > UINT32_MAX)
					{
						tmp32 = UINT32_MAX;
					}
					else if (tmpValue < 0)
					{
						tmp32 = 0;
					}
					else
					{
						tmp32 = (uint32)tmpValue;
					}
					*(uint32*)ioPortData[linkPosition] = tmp32;
				}
				break;
			}
		case 8:
			{
				if (isSigned)
				{
					int64 tmp64;
					if (tmpValue > INT64_MAX)
					{
						tmp64 = INT64_MAX;
					}
					else if (tmpValue < INT64_MIN)
					{
						tmp64 = INT64_MIN;
					}
					else
					{
						tmp64 = (int64)tmpValue;
					}
					*(int64*)ioPortData[linkPosition] = tmp64;
				}
				else
				{
					uint64 tmp64;
					if (tmpValue > UINT64_MAX)
					{
						tmp64 = UINT64_MAX;
					}
					else if (tmpValue < 0)
					{
						tmp64 = 0;
					}
					else
					{
						tmp64 = (uint64)tmpValue;
					}
					*(uint64*)ioPortData[linkPosition] = tmp64;
				}
				break;
			}
		default:
			error = 1;
			break;
		}
	}
	else
	{
		switch (byteCount)
		{
		case 4:
			{
				float32 tmp32;
				if (tmpValue > FLOAT32_MAX)
				{
					tmp32 = FLOAT32_MAX;
				}
				else if (tmpValue < FLOAT32_MIN)
				{
					tmp32 = FLOAT32_MIN;
				}
				else
				{
					tmp32 = (float32)tmpValue;
				}
				*(float32*)ioPortData[linkPosition] = tmp32;
				break;
			}
		case 8:
			{
				*(float64*)ioPortData[linkPosition] = tmpValue;
				break;
			}
		default:
			error = 1;
			break;
		}
	}
	return (error);
}

/* returns outport value */
static uint8 getOutportValue(uint32 tablePosition, float64* value)
{
	uint8 error = 0;
	uint8 isInteger, isSigned, byteCount;
	float64 tmpValue, offset, gain;

	uint32 linkPosition = linkTable[noInports + tablePosition];
	
	isInteger = (ioPortTypes[linkPosition] >> 7) & 0x01;
	isSigned = (ioPortTypes[linkPosition] >> 6) & 0x01;
	byteCount = (ioPortTypes[linkPosition] & 0x3F);

	if (isInteger)
	{
		switch (byteCount)
		{
		case 1:
			if (isSigned)
			{
				tmpValue = (float64)(*(int8*)ioPortData[linkPosition]);
			}
			else
			{
				tmpValue = (float64)(*(uint8*)ioPortData[linkPosition]);
			}
			break;
		case 2:
			if (isSigned)
			{
				tmpValue = (float64)(*(int16*)ioPortData[linkPosition]);
			}
			else
			{
				tmpValue = (float64)(*(uint16*)ioPortData[linkPosition]);
			}
			break;
		case 4:
			if (isSigned)
			{
				tmpValue = (float64)(*(int32*)ioPortData[linkPosition]);
			}
			else
			{
				tmpValue = (float64)(*(uint32*)ioPortData[linkPosition]);
			}
			break;
		case 8:
			if (isSigned)
			{
				tmpValue = (float64)(*(int64*)ioPortData[linkPosition]);
			}
			else
			{
				tmpValue = (float64)(*(uint64*)ioPortData[linkPosition]);
			}
			break;
		default:
			error = 1;
			break;
		}
	}
	else
	{
		switch (byteCount)
		{
		case 4:
			tmpValue = (float64)(*(float32*)ioPortData[linkPosition]);
			break;
		case 8:
			tmpValue = *(float64*)ioPortData[linkPosition];
			break;
		default:
			error = 1;
			break;
		}
	}

	if (error == 0)
	{
		offset = ioPortOffset[linkPosition];
		gain = ioPortGain[linkPosition];
		*value = tmpValue * gain + offset;
	}

	return (error);
}


uint8 receiveInportValues(TCPIPSocket* socket, uint32* statusFlag)
{
	uint8 error;
	uint32 i;

	/* receive status flag */
	error = socket->RecvData((char*)statusFlag, sizeof(*statusFlag));

	/* receive inport values */
	for (i=0;i<noInports;i++)
	{
		float64 tmp64 = 0;
		error |= socket->RecvData((char*)&tmp64, sizeof(tmp64));
		setInportValue(i, tmp64);
	}
	return (error);
}


uint8 sendOutportValues(TCPIPSocket* socket, uint32* statusFlag)
{
	uint8 error;
	uint32 i;

	/* send status flag */
	error = socket->SendData((char*)statusFlag, sizeof(*statusFlag));

	/* send outport values */
	for (i=0;i<noOutports;i++)
	{
		float64 tmp64 = 0;
		error |= getOutportValue(i, &tmp64);
		error |= socket->SendData((char*)&tmp64, sizeof(tmp64));
	}
	return (error);
}

uint8 sendUpdateIntervalTime(TCPIPSocket* socket, double time)
{
	uint8 error;
	error = socket->SendData((char*)&time, sizeof(time));
	return (error);
}

uint8 setInportOrder(TCPIPSocket* socket)
{
	uint8 error;
	uint32 inportListLength, curLabelLength;
	uint32 i;

	/* receive inports order */
	error = socket->RecvInteger((int32&)inportListLength);
	for (i=0;i<inportListLength;i++)
	{
		char* curLabel;

		curLabelLength = 0;
		error |= socket->RecvInteger((int32&)curLabelLength);
		curLabel = (char*)malloc(curLabelLength);
		error |= socket->RecvData(curLabel, curLabelLength);
		error |= linkPort(curLabel, i);
		free(curLabel);
	}
	return (error);
}

uint8 setOutportOrder(TCPIPSocket* socket)
{
	uint8 error;
	uint32 outportListLength, curLabelLength;
	uint32 i;

	/* receive outports order */
	error = socket->RecvInteger((int32&)outportListLength);
	for (i=0;i<outportListLength;i++)
	{
		char* curLabel;

		curLabelLength = 0;
		error |= socket->RecvInteger((int32&)curLabelLength);
		curLabel = (char*)malloc(curLabelLength);
		error |= socket->RecvData(curLabel, curLabelLength);
		error |= linkPort(curLabel, noInports + i);
		free(curLabel);
	}
	return (error);
}
